######################
# Materiały pomocnicze do przedmiotu ZMADBM
# Wyklad 5
# Modelowanie danych i analiza wyników klasyfikacji.
# dr inż. Michal Dramiński
######################

#formula w R
#zmienna predykcyjna (objaśniana) znak tyldy 
#i kropka oznaczająca użyj wszystkich zmiennych wejściowych (objaśniających)
fit <- lm(hp~., mtcars)
# tu przyklad jak uzyc formuly ze stringa
f <- as.formula(paste0("Species","~."))
class(f)

#model liniowy
#pierwszy parametr to formula 
fit <- lm(hp~., mtcars)
fit
plot(fit)
coefficients(fit) # model coefficients
residuals(fit) # residuals
fitted(fit) # predicted values

#wartości wynikające z modelu vs oryginalne
plot(mtcars$hp,fitted(fit))

#regresja logistyczna w pakiecie nnet
library(nnet)
classifier.nnet <- nnet::multinom(Species~., data = iris)
predictions <- as.character(predict(classifier.nnet, iris), type="response")
data.frame(iris$Species,predictions)
table(iris$Species, predictions)

#kNN
library(class)
classifier.knn <- class::knn(train=iris[,-ncol(iris)], test=iris[,-ncol(iris)], cl=iris$Species, k = 3, prob=TRUE)
predictions <- as.character(as.vector(classifier.knn))
table(iris$Species, predictions)

### drzewo decyzyjne rpart
library(rpart)
classifier.rpart <- rpart::rpart(Species~., data = iris)
predictions <- as.character(predict(classifier.rpart,iris,type="class"))
table(iris$Species, predictions)

### las losowy
library(randomForest)
classifier.rf <- randomForest::randomForest(Species~., data = iris)
predictions <- as.character(predict(classifier.rf,iris,type="class"))
table(iris$Species, predictions)

### naiwny klasyfikator bayesowski (naive bayes)
#dziala tylko na danych nominalnych
library(e1071)
classifier.nb <- e1071::naiveBayes(Species~., data = iris, laplace = 0, na.action = na.pass)
predictions <- as.character(predict(classifier.nb,iris),type=c("class", "raw"),threshold = 0.001)
table(iris$Species, predictions)

### siec neuronowa (neural net)
library("nnet")
n <- names(iris)
f <- as.formula(paste("Species ~", paste(n[!n %in% "Species"], collapse = " + ")))
classifier.nn <- nnet(f,data=iris,size=10,linout=T)
predictions <- as.character(predict(classifier.nn,iris,type="class"))
table(iris$Species, predictions)
#show nnet
library(devtools)
source_url('https://gist.githubusercontent.com/fawda123/7471137/raw/466c1474d0a505ff044412703516c34f1a4684a5/nnet_plot_update.r')
plot.nnet(classifier.nn)

### indukcja reguł decyzyjnych (rule induction)
#dziala tylko na danych nominalnych
library("RoughSets")
#numeryczne dane mozna poddac dyskretyzacji
#czyli numeryczne cechy -> nominalne
rule.data <- SF.asDecisionTable(iris, decision.attr = ncol(iris))
cut.values <- D.discretization.RST(rule.data, type.method = "local.discernibility")
rule.data <- SF.applyDecTable(rule.data, cut.values)

#Algorytm Regułowy - AQ
rules <- RI.AQRules.RST(rule.data, confidence = 0.9, timesCovered = 3)
predictions <- predict(rules,rule.data)
table(iris$Species, predictions$predictions)

#Algorytm Regułowy - LEM2
rules <- RI.LEM2Rules.RST(rule.data)
predictions <- predict(rules,rule.data)
table(iris$Species, predictions$predictions)

#Algorytm Regułowy - CN2
rules <- RI.CN2Rules.RST(rule.data, K = 5)
predictions <- predict(rules,rule.data)
table(iris$Species, predictions$predictions)

### Support Vector Machines (SVM)
library(e1071)
classifier.svm <- e1071::svm(Species~., data = iris, cost = 1000, gamma = 0.0001)
predictions <- as.character(predict(classifier.svm,iris),type=c("class", "raw"),threshold = 0.001)
table(iris$Species, predictions)

